@@ -1,35 +1,64 @@
-# $Id: Calc.pm,v 1.21 2008/09/14 12:30:22 cfaerber Exp $
-#
 package Color::Calc;
 
+use attributes;
 use strict;
+use utf8;
 use warnings;
 
 use Carp;
 use Exporter;
 use Params::Validate qw(:all);
-use POSIX;
+use POSIX ();
+
+use Scalar::Util qw(dualvar);
+use List::Util qw(min max reduce sum);
 
 use Graphics::ColorNames qw( hex2tuple tuple2hex );
-use Graphics::ColorNames::HTML;
+use Graphics::ColorNames::HTML qw();
 
-our $VERSION = '1.05';
+our $VERSION = "1.074";
 $VERSION = eval $VERSION;
 
 our $MODE = ();
 
 my %__HTMLColors = ();
-our @__subs = qw( blend blend_bw bw contrast contrast_bw dark get grey gray invert light mix );
+our @__subs = qw(
+  blend blend_bw
+  bw
+  contrast contrast_bw
+  dark
+  get
+  gray
+  grey
+  invert
+  light
+  mix
+  opposite
+  round
+  safe
+);
+
+sub __put_tuple { map { my $a = int($_); length($a) % 3 ? $a : dualvar($a, "0$a") } @_ };
+sub __put_hex 	{ dualvar((reduce { ($a << 8) | ($b & 0xFF) } @_), tuple2hex(@_)) };
+sub __put_html	{ my $col = lc(tuple2hex(@_)); $__HTMLColors{$col} || '#'.$col; };
+sub __put_object{ return Graphics::ColorObject->new_RGB255( \@_, '', '' ); };
+sub __put_obj	{ return Color::Object->newRGB(map { 255*$_; } @_); };
 
 my %__formats = (
-  'tuple'	=> sub { return __normtuple_out(@_); },
-  'hex'		=> sub { return tuple2hex((@_)); },
-  'html'	=> sub { my $col = lc(tuple2hex((@_))); return $__HTMLColors{$col} || '#'.$col; },
-  'object'	=> sub { eval { require Graphics::ColorObject; }; return Graphics::ColorObject->new_RGB255( [(@_)] ); },
-  'obj'		=> sub { eval { require Color::Object; }; return Color::Object->newRGB(map { 255*$_; } @_); },
-  'pdf'		=> sub { return '#'.(tuple2hex(@_)) },
+  map { m/^__put_(.*)/ ? ( $1 => $Color::Calc::{$_} ) : () }
+    keys %Color::Calc::
 );
 
+# use Data::Dumper;
+# print STDERR Dumper(\%__formats);
+
+my %__formats_require = (
+  'obj'		=> 'Color::Object',
+  'object'	=> 'Graphics::ColorObject',
+);
+
+$__formats{'pdf'} = $__formats{'html'};
+
 my @__formats = keys %__formats;
 my $__formats_re = join('|', @__formats,'__MODEvar');
 
@@ -45,7 +74,7 @@ our @EXPORT = ('color', map({"color_$_"} @__formats, map({my $s=$_; (map{$s.'_'.
 our @ISA = ('Exporter');
 
 my %new_param = (
-  'ColorScheme' => { type => SCALAR, optional => 1 },
+  'ColorScheme' => { type => SCALAR | HANDLE | HASHREF | ARRAYREF | CODEREF, optional => 1 },
   'OutputFormat' => { type => SCALAR, untaint => 1, regexp => qr($__formats_re), optional => 1 },
 );
 
@@ -53,14 +82,25 @@ sub new {
   my $pkg = shift; validate(@_, \%new_param);
   my $self = {@_}; bless($self, $pkg);
 
-  if(!ref($self->{'ColorScheme'})) {
+  unless(UNIVERSAL::isa($self->{'ColorScheme'}, 'Graphics::ColorNames')) {
     my %ColorNames;
-    tie %ColorNames, 'Graphics::ColorNames', ($self->{'ColorScheme'} || 'X');
+    if(defined $self->{'ColorScheme'}) {
+      if(!ref $self->{'ColorScheme'} && $self->{'ColorScheme'} =~ m/^([[:alnum:]_]+)$/) {
+        my $module = 'Graphics::ColorNames::'.$1;
+        eval "use $module;"; croak $! if $@;
+        my $names = UNIVERSAL::can($module, 'NamesRgbTable');
+        croak "$module is not compatible with Graphics::ColorNames" if !$names;
+        $self->{'ColorScheme'} = &$names();
+      }
+
+      tie %ColorNames, 'Graphics::ColorNames', $self->{'ColorScheme'};
+    } else {
+      tie %ColorNames, 'Graphics::ColorNames';
+    }
     $self->{'ColorScheme'} = \%ColorNames;
   }
 
   $self->set_output_format($self->{'OutputFormat'} || 'tuple');
-
   return $self;
 }
 
@@ -70,10 +110,10 @@ sub __get_default {
   return $__default_object;
 }
 
-my $__tuple_object = undef;
-sub __get_tuple {
-  $__tuple_object = __PACKAGE__->new('OutputFormat' => 'tuple') unless $__tuple_object;
-  return $__tuple_object;
+my $__raw_object = undef;
+sub __get_raw {
+  $__raw_object = __PACKAGE__->new('OutputFormat' => 'tuple') unless $__raw_object;
+  return $__raw_object;
 }
 
 my %import_param = ( 
@@ -127,17 +167,14 @@ sub __import {
   return 1;
 }
 
+sub __dualvar_tuple {
+  my $str = shift;
+  my $num = reduce { ($a << 8) | ($b & 0xFF) } @_;
+  return dualvar $num, $str;
+}
 
 sub __normtuple_in {
-  return map { ($_ < 0) ? 0 : (($_ > 255) ? 255 : int($_+.5)) } @_;
-}
-
-sub __normtuple_out {
-  return map { (length $_) == 3 ? '0'.$_ : $_ } __normtuple_in(@_)
-}
-
-sub __normtuple {
-  return __normtuple_out(__normtuple_in(@_));
+  return map { (!defined($_) || $_ < 0) ? 0 : (($_ > 255) ? 255 : int($_+.5)) } @_;
 }
 
 sub __is_col_val {
@@ -187,7 +224,7 @@ sub __get {
   }
   else {
     my $col = $self->{'ColorScheme'}->{$$p[0]};
-    if($col) {
+    if(defined $col) {
       shift @$p; 
       return hex2tuple($col);
     } else {
@@ -197,12 +234,23 @@ sub __get {
   }
 }
 
+sub __require_format {
+  my $new_fmt = shift;
+
+  if(exists $__formats_require{$new_fmt}) {
+    eval "use $__formats_require{$new_fmt}()"; 
+    croak $@ if $@;
+  }
+  return 1;
+}
+
 sub set_output_format {
   validate_pos(@_, { isa => __PACKAGE__ }, { type => SCALAR, regexp => qr($__formats_re) });
   my $self = shift;
+  my $new_fmt = shift;
 
   my $old = $self->{'OutputFormat'};
-  $self->{'OutputFormat'} = shift;
+  $self->{'OutputFormat'} = $new_fmt;
 
   $self->{'__put'} = $self->{'OutputFormat'} eq '__MODEvar' 
     ? sub{ return $__formats{$MODE || 'tuple'}->(@_); }
@@ -224,80 +272,6 @@ sub __get_self {
   }
 }
 
-sub get { 
-  my $self = __get_self(\@_); 
-  return $self->__put($self->__get(\@_)); 
-}
-
-sub invert {
-  my $self = __get_self(\@_);
-  return $self->__put(map { 255 - $_ } $self->__get(\@_));
-}
-
-sub bw {
-  my $self = __get_self(\@_);
-  my @c = $self->__get(\@_);
-  my $g = $c[0]*.3 + $c[1]*.59 + $c[2]*.11;
-  return $self->__put(__normtuple($g,$g,$g));
-}
-
-*grey = \&bw;
-*gray = \&bw;
-
-sub mix {
-  my $self = __get_self(\@_);
-  my @c1 = ($self->__get(\@_,1));
-  my @c2 = ($self->__get(\@_));
-  my $alpha = shift(@_); $alpha = 0.5 unless defined $alpha;
-
-  return $self->__put(__normtuple(
-    ($c1[0] + ($c2[0]-$c1[0])*$alpha),
-    ($c1[1] + ($c2[1]-$c1[1])*$alpha),
-    ($c1[2] + ($c2[2]-$c1[2])*$alpha) ));
-}
-
-sub light {
-  my $self = __get_self(\@_);
-  return $self->__put(__get_tuple->mix([$self->__get(\@_)],[255,255,255],shift));
-}
-
-sub dark {
-  my $self = __get_self(\@_);
-  return $self->__put(__get_tuple->mix([$self->__get(\@_)],[0,0,0],shift));
-}
-
-sub contrast {
-  my $self = __get_self(\@_);
-  my @rgb = $self->__get(\@_);
-  my $cut = (shift || .5) * 255;
-  return $self->__put(map { $_ >= $cut ? 0 : '0255' } @rgb);
-}
-
-sub contrast_bw {
-  my $self = __get_self(\@_);
-  my @rgb = $self->__get(\@_);
-  return $self->__put(__get_tuple->contrast([__get_tuple->bw(@rgb)], shift));
-}
-
-sub blend {
-  my $self = __get_self(\@_);
-  my @c1 = $self->__get(\@_);
-  return $self->mix(\@c1,[__get_tuple->contrast(\@c1)],shift);
-}
-
-sub blend_bw {
-  my $self = __get_self(\@_);
-  my @c = $self->__get(\@_);
-  return $self->mix(\@c,[__get_tuple->contrast_bw(\@c)],shift);
-}
-
-foreach my $format (@__formats) {
-  __import(__PACKAGE__, 'Prefix' => 'color', '__Suffix' => "_$format", 'OutputFormat' => $format);
-  __import(__PACKAGE__, 'Prefix' => '',      '__Suffix' => "_$format", 'OutputFormat' => $format);
-}
-
-__import(__PACKAGE__, 'Prefix' => 'color', 'OutputFormat' => '__MODEvar');
-
 =head1 NAME
 
 Color::Calc - Simple calculations with RGB colors.
@@ -313,33 +287,24 @@ Color::Calc - Simple calculations with RGB colors.
 
 =head1 DESCRIPTION
 
-The C<Color::Calc> module implements simple calculations with RGB
-colors. This can be used to create a full color scheme from a few
-colors.
+The C<Color::Calc> module implements simple calculations with RGB colors. This
+can be used to create a full color scheme from a few colors.
 
-=head2 Using the calculation functions
+=head2 USAGE
 
-There are three methods to use the calculation functions: You can
-create an object, import customised functions into your namespace,
-or you can access them as class methods.  
+=head3 Constructors
 
-=head3 Object-oriented interface
+=over
 
-The object-oriented interface can be used to access the calculation functions
-through an object reference:
+=item Color::Calc->new( ... )
+
+This class method creates a new C<Color::Calc> object.
 
   use Color::Calc();
   my $cc = new Color::Calc( 'ColorScheme' => 'X', OutputFormat => 'HTML' );
   print $cc->invert( 'white' );
 
-=over
-
-=item new
-
-  $cc = new Color::Calc( 'ColorScheme' => $name, 'OutputFormat' => $format );
-
-Creates a new C<Color::Calc> object. The constructor can be passed
-the following options:
+It accepts the following parameters:
 
 =over
 
@@ -352,29 +317,67 @@ names defined in the HTML 4.0 specification). For a full list of
 possible values, please refer to the documentation of of
 C<Graphics::ColorNames>.
 
+Unlike C<Graphics::ColorNames>, barewords are I<always> interpreted as a module
+name under C<Graphics::ColorNames>. If you really want to use a filename like
+"foo", you have to write it as "./foo".
+
 Default: C<X> (Note: This is incompatible with HTML color names).
 
 =item OutputFormat
 
-One of the output formats defined by this module. See below for
-possible values.
+One of the output formats defined by this module. Possible values are:
 
-Default: C<__MODEvar>
+=over
+
+=item tuple
+
+Returns a list of three values in the range 0..255. The first value is
+guaranteed to have a C<length> that is not a multiple of three.
+
+=item hex
+
+Returns a hexadecimal RGB value as a scalar that contains a string in the
+format RRGGBB and a number representing the hexadecimal number 0xRRGGBB.
+
+=item html
+
+Returns a string compatible with W3C's HTML and CSS specifications,
+i.e. I<#RRGGBB> or one of the sixteen HTML color names.
+
+=item obj
+	
+(DEPRECATED) Returns a C<Color::Object> reference. The module C<Color::Object>
+must be installed, of course.
+
+=item object
+
+Returns a C<Graphics::ColorObject> reference. The module
+C<Graphics::ColorObject> must be installed, of course.
+
+=item pdf
+
+Returns a string compatible with C<PDF::API2>, i.e. I<#RRGGBB>.
+
+=item __MODEvar
+
+(DEPRECATED) Uses the value of C<$Color::Calc::MODE> to select one
+of the above output formats. You should use C<local> when setting
+this variable:
+
+  local $Color::Calc::MODE = 'html';
 
 =back
 
-=item set_output_format
-
-  $cc->set_output_format( $format );
-
-Changes the output format for an existing C<Color::Calc> object.  
+Default: C<__MODEvar> (for compatibility)
 
 =back
 
-=head3 Importing customised functions
+=item Color::Calc->import( ... )
 
-You can also choose to import customised funtions into your
-namespace:
+This method creates a new, hidden object and binds its methods to the namespace
+of the calling module.
+
+This method is usually not called directly but from perl's C<use> statement:
 
   use Color::Calc(
     'ColorScheme' => 'X',
@@ -396,95 +399,41 @@ See above.
 
 =item Prefix
 
-Adds a prefix to the front of the method names. You can call the
-methods as I<prefix>_I<method_name> (the specified value plus an
-underscore plus the method name). You can also call I<prefix>
-instead of I<prefix>C<_get>.
+Adds a prefix to the front of the method names. The calculation methods are
+bound to the name  I<prefix>_I<method_name> (the specified prefix, an
+underscore, the calculation method's name). Further, I<prefix> is made an alias
+for I<prefix>C<_get>.
 
 Default: C<color>
 
 =back
 
-Please note that specifying an empty list of parameters means
-"don't import anything". Omit the list to import the default
-functions.
+Please note that with perl's C<use> and C<import> statemehts, omitting the list
+and specifying an empty list has different meanings:
 
-You can also C<use> the following modules to import the function
-with pre-defined parameters.
+  use Color::Calc;	# import with default settings (see below)
 
-=over
-
-=item C<use Color::Calc::WWW>
-
-Same as C<use Color::Calc( ColorScheme => 'WWW', OutputFormat => 'html' )>
-
-=item C<use Color::Calc::html>
-
-(DEPRECATED) Same as C<use Color::Calc( OutputFormat => 'html')>
-
-Please note that this only selects HTML as the C<OutputFormat> but
-not as the C<ColorScheme> (which defaults to 'X'), which is
-probably not what you expect. Use C<Color::Calc::WWW> instead.
-
-=item C<use Color::Calc::hex>
-   
-Same as C<use Color::Calc( OutputFormat => 'hex')>
-
-=item C<use Color::Calc::object>
-   
-Same as C<use Color::Calc( OutputFormat => 'object')>
-
-=item C<use Color::Calc::pdf>
-   
-Same as C<use Color::Calc( OutputFormat => 'pdf')>
-
-=item C<use Color::Calc::tuple>
-   
-Same as C<use Color::Calc( OutputFormat => 'tuple')>
+  use Color::Calc();	# don't import anything
 
 =back
 
-=head3 Calling unimported class functions
+=head3 Property "set"/"get" methods
 
-(DEPRECATED) You can also access the methods as class methods of
-the various packages with and without the C<color> prefix:
+These methods are inaccessible without a object reference, i.e. when the
+functions have been C<import>ed.
 
-  use Color::Calc::WWW();
+=over
 
-  print Color::Calc::WWW::color('FFF');		# prints 'white'
-  print Color::Calc::WWW::color_invert('FFF');	# prints 'black'
-  print Color::Calc::WWW::invert('FFF');	# prints 'black'
+=item $cc->set_output_format( $format)
 
-For the main module C<Color::Calc>, you can also add a suffix
-C<_I<output_format>> to select the output format:
+Changes the output format for an existing C<Color::Calc> object.  
 
-  use Color::Calc();
+=back
 
-  print Color::Calc::color_html('FFF');		# prints 'white'
-  print Color::Calc::color_invert_html('FFF');	# prints 'black'
-  print Color::Calc::color_invert_hex('FFF');	# prints '000000'
+=head3 Calculation methods
 
-=head2 Color formats
-
-The module supports different color formats for in- and output.
-
-As a general rule, all input formats are accepted at any time
-regardless of the output format specified by C<OutputFormat>. This
-also means that function calls can be nested easily:
-
-  print color_invert(color_invert('blue'));
-
-Note: Nesting does not work as expected if you specify C<html> as
-the C<OutputFormat> and a C<ColorScheme> incompatible with HTML
-(e.g. C<X>). (This may change in future versions.)
-
-(Note: The module internally only works with color values in the
-integer range 0..255, i.e. 8bit precision. This may change in
-future versions.)
-
-=head3 Input
-
-The module accepts color values in the following formats:
+All calculation methods I<always> accept the following formats for C<$color> or
+C<$color1>/C<$color2>:
 
 =over
 
@@ -516,85 +465,152 @@ A C<Graphics::ColorObject> reference.
 
 =back
 
-=head3 Output
-
-C<Color::Calc> can return colors in the following formats
+The calculation methods can be either accessed through a C<Color::Calc> object
+reference (here: C<$cc>) or through the method names imported by C<import>
+(here using the prefix L<color>).
 
 =over
 
-=item tuple
-
-Returns a list of three values in the range 0..255. The first value is
-guaranteed to have a C<length> that is not a multiple of three.
-
-=item hex
-
-Returns a hexadecimal RGB value as a string in the format RRGGBB.
-
-(Note: future versions may return an object that mimics a string
-instead.)
-
-=item html
-
-Returns a string compatible with W3C's HTML and CSS specifications,
-i.e. I<#RRGGBB> or one of the sixteen HTML color names.
-
-(Note: future versions may return an object that mimics a string
-instead.)
-
-=item object
-
-Returns a C<Graphics::ColorObject> reference. The module
-C<Graphics::ColorObject> must be installed.
-
-=item pdf
-
-Returns a string compatible with C<PDF::API2>, i.e. I<#RRGGBB>.
-
-(Note: future versions may return an object that mimics a string
-instead.)
-
-=item __MODEvar
-
-(DEPRECATED) Uses the value of C<$Color::Calc::MODE> to select one
-of the above output formats. You should use C<local> when setting
-this variable.
-
-=back
-
-=head2 Calculation functions
-
-The module supports the following calculation functions, which can
-be accessed through one of the methods described above:
-
-=over
-
-=item get($color)
+=item $cc->get($color) / color($color)
 
 Returns C<$color> as-is (but in the selected output format). This
 function can be used for color format conversion/normalisation.
 
-=item invert($color)
+=cut
+
+sub get { 
+  my $self = __get_self(\@_); 
+  return $self->__put($self->__get(\@_)); 
+}
+
+=item $cc->invert($color) / color_invert($color)
 
 Returns the inverse of C<$color>.
 
-=item bw($color)
+=cut
 
-=item grey($color) 
+sub invert {
+  my $self = __get_self(\@_);
+  return $self->__put(map { 255 - $_ } $self->__get(\@_));
+}
 
-=item gray($color)
+=item $cc->opposite($color) / color_opposite($color)
+
+Returns a color that is on the opposite side of the color wheel but roughly
+keeps the saturation and lightness.
+
+=cut 
+
+sub opposite {
+  my $self = __get_self(\@_);
+  my @rgb = $self->__get(\@_);
+
+  my $min = min @rgb;
+  my $max = max @rgb;
+
+  return $self->__put(
+    map { $max - $_ + $min } @rgb
+  );
+}
+
+=item $cc->bw($color) / color_bw($color)
+
+=item $cc->grey($color) / color_grey($color)
+
+=item $cc->gray($color) / color_gray($color)
 
 Converts C<$color> to greyscale.
 
-=item mix($color1, $color2 [, $alpha])
+=cut
+
+sub bw {
+  my $self = __get_self(\@_);
+  my @c = $self->__get(\@_);
+  my $g = $c[0]*.3 + $c[1]*.59 + $c[2]*.11;
+  return $self->__put($g, $g, $g);
+}
+
+*grey = \&bw;
+*gray = \&bw;
+
+=item $cc->round($color, $value_count) / color_round($color, $value_count)
+
+Rounds each component to to the nearest number determined by dividing the range
+0..255 into C<$value_count>+1 portions.
+
+The default for C<$value_count> is 6, yielding S<6^3 = 216> colors.  Values
+that are one higher than divisors of 255 yield the best results (e.g. 3+1, 5+1,
+7+1, 9+1, 15+1, 17+1, ...).
+
+=cut 
+
+sub round {
+  my $self = __get_self(\@_);
+  my @rgb = $self->__get(\@_);
+  my $steps = shift || 6;
+  $steps--;
+
+  return $self->__put(
+    map { int(int( $_ * $steps / 255 + 0.5) * 255 / $steps + 0.5) } @rgb
+  );
+}
+
+=item $cc->safe($color) / color_safe($color)
+
+Rounds each color component to a multiple of 0x33 (dec. 51) or to a named color
+defined in the HTML 4.01 specification.
+
+Historically, these colors have been known as web-safe colors. They still
+provide a convenient color palette.
+
+=cut
+
+sub __dist2 {
+  my @a = splice @_, 0, 3;
+  return sum map { POSIX::pow($_ - shift @a, 2) } @_;
+}
+
+sub safe {
+  my $self = __get_self(\@_);
+  my @rgb = $self->__get(\@_);
+
+  my @new_rgb = __get_raw->round(@rgb);
+  my $new_d2 = __dist2(@rgb, @new_rgb);
+
+  foreach my $h (keys %__HTMLColors) {
+    my @h_rgb = hex2tuple($h);
+    my $h_d2 = __dist2(@rgb, @h_rgb);
+
+    if($h_d2 <= $new_d2) {
+      @new_rgb = @h_rgb;
+      $new_d2 = $h_d2;
+    }
+  }
+  return $self->__put(@new_rgb);
+}
+
+=item $cc->mix($color1, $color2 [, $alpha]) / color_mix($color1, $color2 [, $alpha])
 
 Returns a color that is the mixture of C<$color1> and C<$color2>.
 
 The optional C<$alpha> parameter can be a value between 0.0 (use
-C<$color1> only) and 1.0 (use C<$color2> only), the default is
-0.5.
+C<$color1> only) and 1.0 (use C<$color2> only), the default is 0.5.
 
-=item light($color [, $alpha])
+=cut
+
+sub mix {
+  my $self = __get_self(\@_);
+  my @c1 = ($self->__get(\@_,1));
+  my @c2 = ($self->__get(\@_));
+  my $alpha = shift(@_); $alpha = 0.5 unless defined $alpha;
+
+  return $self->__put(
+    ($c1[0] + ($c2[0]-$c1[0])*$alpha),
+    ($c1[1] + ($c2[1]-$c1[1])*$alpha),
+    ($c1[2] + ($c2[2]-$c1[2])*$alpha) );
+}
+
+=item $cc->light($color [, $alpha]) / color_light($color [, $alpha])
 
 Returns a lighter version of C<$color>, i.e. returns
 C<mix($color,[255,255,255],$alpha)>.
@@ -602,7 +618,14 @@ C<mix($color,[255,255,255],$alpha)>.
 The optional C<$alpha> parameter can be a value between 0.0 (use C<$color>
 only) and 1.0 (use [255,255,255] only), the default is 0.5.
 
-=item dark($color [, $alpha])
+=cut
+
+sub light {
+  my $self = __get_self(\@_);
+  return $self->__put(__get_raw->mix([$self->__get(\@_)],[255,255,255],shift));
+}
+
+=item $cc->dark($color [, $alpha]) / color_dark($color [, $alpha])
 
 Returns a darker version of C<$color>, i.e. returns
 C<mix($color,[0,0,0],$alpha)>.
@@ -610,7 +633,15 @@ C<mix($color,[0,0,0],$alpha)>.
 The optional C<$alpha> parameter can be a value between 0.0 (use
 C<$color> only) and 1.0 (use [0,0,0] only), the default is 0.5.
 
-=item contrast($color [, $cut])
+=cut
+
+sub dark {
+  my $self = __get_self(\@_);
+  return $self->__put(__get_raw->mix([$self->__get(\@_)],[0,0,0],shift));
+}
+
+
+=item $cc->contrast($color [, $cut]) / color_contrast($color [, $cut])
 
 Returns a color that has the highest possible contrast to the input
 color.
@@ -621,7 +652,16 @@ to 255 otherwise.
 
 The default for C<$cut> is .5, representing a cutoff between 127 and 128.
 
-=item contrast_bw($color [, $cut])
+=cut
+
+sub contrast {
+  my $self = __get_self(\@_);
+  my @rgb = $self->__get(\@_);
+  my $cut = (shift || .5) * 255;
+  return $self->__put(map { $_ >= $cut ? 0 : 255 } @rgb);
+}
+
+=item $cc->contrast_bw($color [, $cut]) / color_contrast_bw($color [, $cut])
 
 Returns black or white, whichever has the higher contrast to C<$color>.
 
@@ -630,7 +670,15 @@ above C<($cut * 255)> and white otherwise.
 
 The default for C<$cut> is .5, representing a cutoff between 127 and 128.
 
-=item blend($color [, $alpha])
+=cut
+
+sub contrast_bw {
+  my $self = __get_self(\@_);
+  my @rgb = $self->__get(\@_);
+  return $self->__put(__get_raw->contrast([__get_raw->bw(@rgb)], shift));
+}
+
+=item $cc->blend($color [, $alpha]) / color_blend($color [, $alpha])
 
 Returns a color that blends into the background, i.e. it returns
 C<mix($color,contrast($color),$alpha)>.
@@ -646,41 +694,99 @@ them returns a color somewhere between them.
 You might want to use C<mix($color, $background, $alpha)> instead
 if you know the real background color.
 
-=item blend_bw($color [, $alpha])
+=cut
+
+sub blend {
+  my $self = __get_self(\@_);
+  my @c1 = $self->__get(\@_);
+  return $self->mix(\@c1,[__get_raw->contrast(\@c1)],shift);
+}
+
+=item $cc->blend_bw($color [, $alpha]) / color_blend_bw($color [, $alpha])
 
 Returns a mix of C<$color> and black or white, whichever has the
 higher contrast to C<$color>.
 
 The optional C<$alpha> parameter can be a value between 0.0 (use
-C<$color> only) and 1.0 (use black/white only), the default is
-0.5.
+C<$color> only) and 1.0 (use black/white only), the default is 0.5.
+
+=cut
+
+sub blend_bw {
+  my $self = __get_self(\@_);
+  my @c = $self->__get(\@_);
+  return $self->mix(\@c,[__get_raw->contrast_bw(\@c)],shift);
+}
 
 =back
 
+=head3 Functions
+
+The calculation methods are also available as functions. The output format is
+selected through the function name.
+
+These functions are deprecated as they do not allow selecting the scheme of
+recognized color names, which defaults to L<Graphics::ColorNames::X> (and is
+incompatible with HTML's color names).
+
+By default, i.e. when no list is specified with C<use> or C<import>, all of
+these functions are exported.
+
+=over
+
+=item color, color_mix, ...
+
+Use C<$Color::Calc::MODE> as the output format. This is the default.
+
+=item color_hex, color_mix_html, ...
+
+Use C<hex> as the output format.
+
+=item color_html, color_mix_html, ...
+
+Use C<html> as the output format. Please note that the color names recognized
+are still based on X's color names, which are incompatible with HTML. You can't
+use the output of these functions as input for other color_*_html functions.
+
+See L<Color::Calc::WWW> for an alternative that does not suffer from this
+problem.
+
+=item color_pdf, color_mix_pdf, ...
+
+Use C<pdf> as the output format.
+
+=item color_object, color_mix_object, ...
+
+Use C<object> as the output format.
+
+=back
+
+=cut
+
+foreach my $format (@__formats) {
+  next if !eval{__require_format($format)};
+  __import(__PACKAGE__, 'Prefix' => 'color', '__Suffix' => "_$format", 'OutputFormat' => $format);
+  __import(__PACKAGE__, 'Prefix' => '',      '__Suffix' => "_$format", 'OutputFormat' => $format);
+}
+
+__import(__PACKAGE__, 'Prefix' => 'color', 'OutputFormat' => '__MODEvar');
+
 =head1 SEE ALSO
 
-L<Graphics::ColorNames> (required);
+L<Graphics::ColorNames> (required); L<Graphics::ColorObject> (optional)
 
-L<Graphics::ColorObject> (optional)
+=head1 AUTHOR
 
-=head1 AUTHOR/LICENSE
+Claus FE<auml>rber <CFAERBER@cpan.org>
 
-Copyright  2004-2008 Claus Frber <CFAERBER@cpan.org>
+=head1 LICENSE
 
-This module is free software; you can redistribute it and/or
-modify it under the terms of either the GNU General Public License
-as published by the Free Software Foundation; either version 1, or
-(at your option) any later version, or the "Artistic License".
+Copyright 2004-2010 Claus FE<auml>rber. All rights reserved.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-MA02110-1301, USA.
+This library is free software; you can redistribute it and/or modify it under
+the same terms as Perl itself.
 
 =cut
 
 1;
+__END__
